#include "Command.h"
#include "DataStructures.h"



/*
** Each of the following sets of Execute/Unexecute commands
** correspond to an "undoable" action.
*/


// --------------------------------------------------------------

bool UC_ResizeDiameter::Execute()
{
	// Get root instance
	ApplicationData *app = m_data->GetApplicationData();
	Experiment *exp = m_data->GetExperiment();
	int session = app->GetSession();
	Tube *tube = exp->GetTube(app->GetTube());
	if(tube == NULL) return false;
	Root *root = tube->GetRoot( m_root_id );
	if(root == NULL) return false;
	RootInstance *root_inst = root->GetRootInstance(app->GetSession());
	if(root_inst == NULL) return false;

	return root_inst->ResizeDiameter(m_index, m_new_radius);
}


bool UC_ResizeDiameter::Unexecute()
{
	// Get root instance
	ApplicationData *app = m_data->GetApplicationData();
	Experiment *exp = m_data->GetExperiment();
	int session = app->GetSession();
	Tube *tube = exp->GetTube(app->GetTube());
	if(tube == NULL) return false;
	Root *root = tube->GetRoot( m_root_id );
	if(root == NULL) return false;
	RootInstance *root_inst = root->GetRootInstance(app->GetSession());
	if(root_inst == NULL) return false;

	return root_inst->ResizeDiameter(m_index, m_old_radius);

}

// --------------------------------------------------------------

bool UC_SetRootAlive::Execute()
{
	ApplicationData *app = m_data->GetApplicationData();
	Experiment *exp = m_data->GetExperiment();
	int session = m_session;
  
	Tube *tube = exp->GetTube(app->GetTube());
	/// Failure if the tube is null
	if(tube == NULL) return false;
	Root *root = tube->GetRoot( m_rootid );
	/// Failure if the root is null
	if(root == NULL) return false;
	RootInstance *inst = root->GetRootInstance(app->GetSession());
	/// Failure if the root instance is null
	if(inst == NULL) return false;
	/// If currently alive and to be set dead
	if(!m_alive)
	{       
		// sets all instances dead in current and future sessions
		while(session <= exp->GetMaxSessions())
		{
			inst = root->GetRootInstance(session);
			if (inst != NULL && inst->IsAlive())   inst->SetDead();
			session++;
		}
	} 
	/// If currently set dead and to be set alive
	else 
	{
		// sets all instances alive in current and past sessions
		while(session > 0)
		{
			inst = root->GetRootInstance(session);
			if (inst != NULL && !inst->IsAlive())  inst->SetAlive();
			session--;    
		}
	}  

	return true;
}

bool UC_SetRootAlive::Unexecute()
{
	ApplicationData *app = m_data->GetApplicationData();
	Experiment *exp = m_data->GetExperiment();
	int session = m_session;
  
	Tube *tube = exp->GetTube(app->GetTube());
	/// Failure if the tube is null
	if(tube == NULL) return false;
	Root *root = tube->GetRoot( m_rootid );
	/// Failure if the root is null
	if(root == NULL) return false;
	RootInstance *inst = root->GetRootInstance(app->GetSession());
	/// Failure if the root instance is null
	if(inst == NULL) return false;
	/// If currently alive and to be set dead
	if(m_alive)
	{       
		// sets all instances dead in current and future sessions
		while(session <= exp->GetMaxSessions())
		{
			inst = root->GetRootInstance(session);
			if (inst != NULL && inst->IsAlive())   inst->SetDead();
			session++;
		}
	} 
	/// If currently set dead and to be set alive
	else 
	{
		// sets all instances alive in current and past sessions
		while(session > 0)
		{
			inst = root->GetRootInstance(session);
			if (inst != NULL && !inst->IsAlive())  inst->SetAlive();
			session--;    
		}
	}  

	return true;
}

// --------------------------------------------------------------
bool UC_MoveRootSess::Execute()
{
  // if this is a Redo action, execute the following
  // move actions
  if(m_executed_once)
  {
	m_root_inst = RootSessGetRootInstance( m_root_id );
	if( m_root_inst == NULL ) return false;
  
	// Move each point on the root to its original location using
  // x and y offsets 
  for(int i = 0;i<m_root_inst->GetAllPoints()->size();i++)
	 RootSessMovePoint(m_root_inst, i, m_root_id, m_root_inst->GetPoint(i)->x+m_offset_x,m_root_inst->GetPoint(i)->y+m_offset_y);//may need to update this to be a bit different with x and y
  
	// Move Diameter (assumes only one diameter)
	RootSessMoveDiameter(m_root_inst, 0,m_root_id, m_root_inst->GetDiameter(0)->r_point.x + m_offset_x, m_root_inst->GetDiameter(0)->r_point.y + m_offset_y);
  }
  else
	  m_executed_once = 1;

  return true;
}


bool UC_MoveRootSess::Unexecute()
{

  m_root_inst = RootSessGetRootInstance( m_root_id );
  if( m_root_inst == NULL ) return false;
  
  // Move each point on the root to its original location using
  // x and y offsets 
  for(int i = 0;i<m_root_inst->GetAllPoints()->size();i++)
    RootSessMovePoint(m_root_inst, i, m_root_id, m_root_inst->GetPoint(i)->x-m_offset_x,m_root_inst->GetPoint(i)->y-m_offset_y);//may need to update this to be a bit different with x and y
  
  /// Move Diameter (assumes only one diameter)
  RootSessMoveDiameter(m_root_inst, 0,m_root_id, m_root_inst->GetDiameter(0)->r_point.x - m_offset_x, m_root_inst->GetDiameter(0)->r_point.y - m_offset_y);

  return true;
}


// --------------------------------------------------------------

bool UC_MoveDiameter::Execute()
{
   
  bool success1, success2;
  
  m_root_inst = MoveDiameterGetRootInstance(m_root_id);
 
  /// Failure if root instance is null
  if(m_root_inst == NULL)     return false;
 
  // if we are able to successfully move the diameter back to the
  // right window and the right location within that window, then
  // the move function is a success.
  if(m_root_inst->MoveDiameter(m_index, m_new_x, m_new_y)) success1 = true;
  else success1 = false;

  if(m_root_inst->ChangeWindowOfDiameter(m_index, m_new_window)) success2 = true;
  else success2 = false;

  return success1 && success2;
}


bool UC_MoveDiameter::Unexecute()
{
 
  bool success1, success2;
  
  m_root_inst = MoveDiameterGetRootInstance(m_root_id);
  
  /// Failure if root instance is null
  if(m_root_inst == NULL)     return false;
  
  // if we are able to successfully move the diameter back to the
  // right window and the right location within that window, then
  // the move function is a success.
  if(m_root_inst->MoveDiameter(m_index, m_old_x, m_old_y)) success1 = true;
  else success1 = false;

  if(m_root_inst->ChangeWindowOfDiameter(m_index, m_old_window)) success2 = true;
  else success2 = false;

  return success1 && success2;
}


// --------------------------------------------------------------

bool UC_MoveRootsInWindow::Execute()
{
  if(executed_once)
  {
	//get all roots in tube/window/session
	Experiment *exp = m_data->GetExperiment();  
	Tube *tube = exp->GetTube(m_data->GetApplicationData()->GetTube());
  
	/// Failure if tube is null
	if( tube == NULL ) return false;
  
	/// Get all roots
	vector<Root> *roots = tube->GetRootsInWindow(m_window);
	if (roots)
	{
		vector<Root>::iterator iter;
		
    //for each root, move by the x and y offsets
		for(iter = roots->begin();iter != roots->end();iter++)
			RootWindowMoveRootSess(iter->GetID(),m_data->GetApplicationData()->GetSession(), m_offset_x, m_offset_y);
	}
  }
  else
	  executed_once = 1;

  return true;
}


bool UC_MoveRootsInWindow::Unexecute()
{
  //get all roots in tube/window/session
  Experiment *exp = m_data->GetExperiment();  
  Tube *tube = exp->GetTube(m_data->GetApplicationData()->GetTube());
  
  /// Failure if tube is null
  if( tube == NULL ) return false;
  
  /// Get all roots
  vector<Root> *roots = tube->GetRootsInWindow(m_window);
  if (roots)
  {
    vector<Root>::iterator iter;

    //for each root, move the root back to its original location
    for(iter = roots->begin();iter != roots->end();iter++)
      RootWindowMoveRootSess(iter->GetID(),m_data->GetApplicationData()->GetSession(), -m_offset_x, -m_offset_y);
  }

  return true;
}


// --------------------------------------------------------------

bool UC_SetRootColor::Execute() 
{
  ApplicationData *app = m_data->GetApplicationData();
  Experiment *exp = m_data->GetExperiment();
  
  Tube *tube = exp->GetTube(app->GetTube());
  /// Failure if the tube is null
  if(tube == NULL) return false;
  Root *root = tube->GetRoot( m_root_id );
  /// Failure if the root is null
  if(root == NULL) return false;
  RootInstance *inst = root->GetRootInstance(app->GetSession());
  /// Failure if the root instance is null
  if(inst == NULL) return false;

  m_old_color = inst->GetColor();
  if(m_old_color != m_color_choice)
  {
    /// Set status unsaved
    inst->SetColor(m_color_choice);
	return true;

  }
  else return false;

}

bool UC_SetRootColor::Unexecute() 
{
  ApplicationData *app = m_data->GetApplicationData();
  Experiment *exp = m_data->GetExperiment();
  
  Tube *tube = exp->GetTube(app->GetTube());
  /// Failure if the tube is null
  if(tube == NULL) return false;
  Root *root = tube->GetRoot( m_root_id );
  /// Failure if the root is null
  if(root == NULL) return false;
  RootInstance *inst = root->GetRootInstance(app->GetSession());
  /// Failure if the root instance is null
  if(inst == NULL) return false;


  inst->SetColor(m_old_color);
  return true;
}


// --------------------------------------------------------------
bool UC_MovePoint::Execute()
{

	m_root_inst = MovePointGetRootInstance(m_root_id);
/*
  // store the location of point for Undo
  m_point = m_root_inst->GetPoint(m_index);
  m_old_x = m_point->x;
  m_old_y = m_point->y;

  // get the location of the diameter and store it
  // in a temporary variable
  m_diameter = m_root_inst->GetDiameter(0);
  int temp_diam_x = m_diameter->r_point.x;
  int temp_diam_y = m_diameter->r_point.y;
*/


	/// Failure if root instance is null
	if(m_root_inst == NULL)     return false;
/*
  // if in Redo, set the location of the diameter 
  // to the previous value
  if (m_old_x > 0 && m_old_y > 0 )
  {
    m_root_inst->MoveDiameter(0, m_diam_x, m_diam_y);
    
    // change the window of the point
    m_root_inst->ChangeWindowOfPoint( m_index, m_point->window );

  }
*/


	m_root_inst->MoveDiameter(0, m_new_diam_x, m_new_diam_y);

	m_root_inst->ChangeWindowOfPoint( m_index, m_new_window );
	
	/// Move the point
    if(m_root_inst->MovePoint(m_index, m_new_x, m_new_y)) return true;
    else return false;
/*
  m_diam_x = temp_diam_x;
  m_diam_y = temp_diam_y;

  
  /// Move the point
  if(m_root_inst->MovePoint(m_index, m_new_x, m_new_y)) return true;
  else return false;
*/

}

bool UC_MovePoint::Unexecute()
{
  m_root_inst = MovePointGetRootInstance(m_root_id);

  // now, move the diameter back to its original position
  m_root_inst->MoveDiameter(0, m_old_diam_x, m_old_diam_y); 


  m_root_inst->ChangeWindowOfPoint( m_index, m_old_window );
/*
  // store the location of the point for Redo
  m_point = m_root_inst->GetPoint(m_index);
  m_new_x = m_point->x;
  m_new_y = m_point->y;


  // remember the location of the diameter
  m_diameter = m_root_inst->GetDiameter(0);

  // store these in temporary variables
  int temp_diam_x = m_diameter->r_point.x;
  int temp_diam_y = m_diameter->r_point.y;

  // now, move the diameter back to its original position
  m_root_inst->MoveDiameter(0, m_diam_x, m_diam_y); 
  
  // change window of point
  m_root_inst->ChangeWindowOfPoint( m_index, m_point->window );

  // remember the location of the diameter for the
  // Redo operation. store in the m_diam variables.
  m_diam_x = temp_diam_x;
  m_diam_y = temp_diam_y;
*/

  /// Move the point
  if(m_root_inst->MovePoint(m_index, m_old_x, m_old_y)) return true;
  else return false;

}

// --------------------------------------------------------------
bool UC_AddPoint::Execute()
{
  m_root_inst = AddPointGetRootInstance(m_root_id);
  
  /// Failure if root instance is null
  if(m_root_inst == NULL) return false;
  
  m_point = m_root_inst->CreateMidPoint(m_index);
  
  // get the location of the diameter
  m_diameter = m_root_inst->GetDiameter(0);
  
  // store the location in a temporary variable before
  // it gets changed by the MoveDiameter operation
  int temp_diam_x = m_diameter->r_point.x;
  int temp_diam_y = m_diameter->r_point.y;


  if(m_temp_point.x > 0 && m_temp_point.y > 0 && m_temp_point.window > 0)
  {
    // assign coordinates to the new point using the previous location
    m_point->x = m_temp_point.x;
    m_point->y = m_temp_point.y;
    m_point->window = m_temp_point.window;

    // move the diameter to the previous location, stored
    // in the m_diam variables
    m_root_inst->MoveDiameter(0, m_diam_x, m_diam_y);
    
  }

  // to Undo this, we will need the original location in Unexecute
  // store this location in the m_diam variables
  m_diam_x = temp_diam_x;
  m_diam_y = temp_diam_y;

  
  return true;
}

bool UC_AddPoint::Unexecute()
{
  m_root_inst = AddPointGetRootInstance(m_root_id);
  
  /// Failure if root instance is null
  if(m_root_inst == NULL) return false;

  // remember the final location of the newly added point
  m_point = m_root_inst->GetPoint(m_index+1);

  m_temp_point.x = m_point->x;
  m_temp_point.y = m_point->y;
  m_temp_point.window = m_point->window;

  // delete the point.
  m_root_inst->DeletePoint(m_index+1);
  
  // remember the location of the diameter
  m_diameter = m_root_inst->GetDiameter(0);

  // store these in temporary variables
  int temp_diam_x = m_diameter->r_point.x;
  int temp_diam_y = m_diameter->r_point.y;

  // now, move the diameter back to its original position
  m_root_inst->MoveDiameter(0, m_diam_x, m_diam_y); 
  
  // remember the location of the diameter for the
  // Redo operation. store in the m_diam variables.
  m_diam_x = temp_diam_x;
  m_diam_y = temp_diam_y;


  return true;
}


// --------------------------------------------------------------
bool UC_DeletePoint::Execute()
{
  m_root_inst = DeletePointGetRootInstance(m_root_id);
  
  /// Failure if root instance is null
  if(m_root_inst == NULL) return false;
    
  
  // delete has to be undoable. remember the current
  // location of the diameter so that we can revert to it.
  m_diameter = m_root_inst->GetDiameter(0);

  // store these in temporary variables
  int temp_diam_x = m_diameter->r_point.x;
  int temp_diam_y = m_diameter->r_point.y;

  // move diameter to original position. applicable only for a Redo
  // operation. the if statement checks if this is a Redo
  if(m_temp_point.x > 0 && m_temp_point.y > 0 && m_temp_point.window > 0)
  {
    m_root_inst->MoveDiameter(0, m_diam_x, m_diam_y);
  }
  
  // remember the location of the diameter for the
  // Redo operation. store in the m_diam variables.
  m_diam_x = temp_diam_x;
  m_diam_y = temp_diam_y;

  // store point coordinates for the Undo operation
  m_point = m_root_inst->GetPoint(m_index);
  m_temp_point.x = m_point->x;
  m_temp_point.y = m_point->y;
  m_temp_point.window = m_point->window;
 
  // this is where the point gets deleted
  m_root_inst->DeletePoint(m_index);

  // Note: DeletePoint() from DataStructures.cpp returns TRUE
  // if the erase operator from Vector returns NULL. But erase
  // returns the new location of the next element, which is not
  // supposed to be NULL. - Ninad Pradhan (04/24/08)

  return true;
}

bool UC_DeletePoint::Unexecute()
{
  m_root_inst = DeletePointGetRootInstance(m_root_id);
  
  /// Failure if root instance is null
  if(m_root_inst == NULL) return false;

  // create a point
  m_point = m_root_inst->CreateMidPoint(m_index-1);

  // assign the previously stored coordinates to this
  // point
  m_point->x = m_temp_point.x;
  m_point->y = m_temp_point.y;
  m_point->window = m_temp_point.window;

  // store location of the diameter so that we can revert to it.
  m_diameter = m_root_inst->GetDiameter(0);

  // store these in temporary variables
  int temp_diam_x = m_diameter->r_point.x;
  int temp_diam_y = m_diameter->r_point.y;

  // move diameter to original position. 
  m_root_inst->MoveDiameter(0, m_diam_x, m_diam_y);
  
  // remember the location of the diameter for the
  // Undo operation. store in the m_diam variables.
  m_diam_x = temp_diam_x;
  m_diam_y = temp_diam_y;


  return true;
}

// --------------------------------------------------------------

bool UC_CreateRoot::Execute() 
{
  Experiment *exp = m_data->GetExperiment();
  Tube* tube = exp->GetTube(m_tube_tag);
  Root *temp_root = NULL;

    /// If tube does not exist, create a new tube
  if (tube == NULL)  tube = exp->CreateTube(m_tube_tag);
  assert(tube);

  if(!m_executed)
  {
    temp_root = tube->CreateRoot(m_primary_window);
    m_root_id = temp_root->GetID();
	m_executed = true;
  }
  else
  {
    temp_root = tube->CreateRoot(m_primary_window, m_root_id);
  }

  RootInstance* ri = temp_root->CreateRootInstance(m_session);
  ri->SetConditionCode(m_default_cond);
  *ri = m_ri_orig;
  

  return true;
}

bool UC_CreateRoot::Unexecute() 
{

  Experiment *exp = m_data->GetExperiment();
  Tube* tube = exp->GetTube(m_tube_tag);

    /// If tube does not exist, create a new tube
  if (tube == NULL)  tube = exp->CreateTube(m_tube_tag);
  assert(tube);

  
  return tube->DeleteRoot(m_root_id);

}

// --------------------------------------------------------------

bool UC_DeleteRoot::Execute() 
{
  Experiment *exp = m_data->GetExperiment();
  Tube* tube = exp->GetTube(m_tube_tag);
  /// Failure if tube is null
  if(tube == NULL) {
    return false;
  }
  assert(tube);
  
  return tube->DeleteRoot(m_root_id);
}

bool UC_DeleteRoot::Unexecute() 
{
  Experiment *exp = m_data->GetExperiment();
  Tube* tube = exp->GetTube(m_tube_tag);
  Root *new_root = NULL;

  // If tube does not exist, create a new tube
  if (tube == NULL)  tube = exp->CreateTube(m_tube_tag);
  assert(tube);

  new_root = tube->CreateRoot(m_primary_window, m_root_id);

  *new_root = m_root_orig;

  return true;
}

/*Implementation for C_CreatePoint*/
bool UC_CreatePoint::Execute()
{
  
  /// Failure if root instance is null
  if(m_root_inst == NULL)
  {
    return NULL;
  }
  
  m_temp_point = m_root_inst->CreatePoint();
  return true;
  //return temp_point;
}

bool UC_CreatePoint::Unexecute()
{
	int i; 
	if(m_root_inst == NULL)
	{
		return NULL;
	 }
	std::vector<RPoint>* point = m_root_inst->GetAllPoints();
	for(i = 0; ; i++)
		if(m_temp_point->x == point->at(i).x && m_temp_point->y == point->at(i).y &&
				m_temp_point->window == point->at(i).window) break;
	return m_root_inst->DeletePoint(i);
}

/*Implementation for C_CopySession*/

bool UC_CopySession::Execute()
{
  Experiment *exp = m_data->GetExperiment();
  Tube *tube = exp->GetTube(m_tubename);
  vector<Tube> *alltubes;
  vector<Root> *roots;
  vector<RootInstance> *instances;
  bool copy_all = (m_curr_window == -1);

  bool notified=false;
  int ok_to_copy = true;    // This is set to false later only if user cancels operation based on message box
  int i,j,k,m;
  
  /// Failure if tube is null
  if(tube == NULL)  return false;
  
  //Check the bounds of the source and destination sessions
  assert(m_from >= 1 && m_to >= 1);
  if (m_from < 1 || m_to < 1) 
  {
    AfxMessageBox("Source and destination sessions must be positive.  Unable to copy.", MB_ICONINFORMATION);
    return false;
  }
  assert(m_from + 1 <= m_to);
  if (m_from + 1 > m_to) 
  {
    AfxMessageBox("Source session must be less than destination session.  Unable to copy.", MB_ICONINFORMATION);
    return false;
  }
  
  alltubes = exp->GetAllTubes();
  

  //Check to see if roots exist in destination sessions.  If they are, prompt the user to see
  //if they want them overwritten
  for (i=0; i<alltubes->size(); i++) 
  {
    roots = alltubes->at(i).GetAllRoots();
	m_roots = alltubes->at(i).GetAllRoots();
    
    //Loop through ALL roots in the current tube.
    for (j=0; j<roots->size(); j++)
    {
      if(copy_all || (roots->at(j).GetPrimaryWindow() == m_curr_window && tube->GetTag() == alltubes->at(i).GetTag()))
      {
        instances = roots->at(j).GetAllRootInstances();
		//m_roots = roots->at(j).GetAllRootInstances();
        //Loop through ALL sessions in the current root to determine if there already exists a session
        //in the selected session interval
        for( k=0; k<instances->size(); k++)
        {
          for( m= m_from+1; m <= m_to; m++)
          {
            // If the session number of this root instance is equal to l, which is in the interval from "from" to "to"
            //  ask the user if they want to overwrite the destination sessions.
            if( instances->at(k).GetSessionNumber() == m && !notified)
            {
              CString str;
              if (copy_all)
              {
                if (m_from+1 == m_to)
                {
                  //str.Format("Session %d already contains roots.\r\nAre you sure you want to overwrite this data?", m_to);
                }
                else
                {
                  //str.Format("Sessions %d through %d already contain roots.\r\nAre you sure you want to overwrite this data?", m_from+1, m_to);
                }
              }
              else
              {
                if (m_from+1 == m_to)
                {
                  //str.Format("Window %d of Session %d already contains roots.\r\nAre you sure you want to overwrite this data?", m_curr_window, m_to);
                }
                else
                {
                  //str.Format("Window %d of Sessions %d through %d already contain roots.\r\nAre you sure you want to overwrite this data?", m_curr_window, m_from+1, m_to);
                }
              }
              DWORD choice = AfxMessageBox(str, MB_YESNO | MB_DEFBUTTON2);
              if (choice != IDYES)  ok_to_copy = false;
              notified = true;
            }
          }
        }
      }
    }
  }


  //If the user chose to overwrite (or wasn't prompted at all) continue
  if (ok_to_copy) 
  {
    //Copy over all tubes and windows
    if (copy_all)
    {
      for (i=0; i<alltubes->size();i++)
      {
        roots = alltubes->at(i).GetAllRoots();
        for (j=0; j < roots->size(); j++) 
        {
          for (k=m_from+1; k <=m_to; k++) 
          {
            roots->at(j).DeleteRootInstance(k);
            roots->at(j).CopyRootInstance(m_from, k);
          }
        }
      }      
    }
    //else only copy within the current window
    else 
    {
      roots = tube->GetAllRoots();
      for(int i = 0; i < roots->size(); i++)
      {
        if(roots->at(i).GetPrimaryWindow() == m_curr_window)
        {   
          for (k=m_from+1; k<=m_to;k++) 
          {
            roots->at(i).DeleteRootInstance(k);
            roots->at(i).CopyRootInstance(m_from, k);
          }
        }
      }
    }
  }
  return true;
}


bool UC_CopySession::Unexecute()
{
	Experiment *exp = m_data->GetExperiment();
  Tube *tube = exp->GetTube(m_tubename);
  vector<Tube> *alltubes;
  vector<Root> *roots;
  vector<RootInstance> *instances;
  bool copy_all = (m_curr_window == -1);

  bool notified=false;
  int ok_to_copy = true;    // This is set to false later only if user cancels operation based on message box
  int i,j,k,m;
  
  /// Failure if tube is null
  if(tube == NULL)  return false;
  
  //Check the bounds of the source and destination sessions
  assert(m_from >= 1 && m_to >= 1);
  if (m_from < 1 || m_to < 1) 
  {
    AfxMessageBox("Source and destination sessions must be positive.  Unable to copy.", MB_ICONINFORMATION);
    return false;
  }
  assert(m_from + 1 <= m_to);
  if (m_from + 1 > m_to) 
  {
    AfxMessageBox("Source session must be less than destination session.  Unable to copy.", MB_ICONINFORMATION);
    return false;
  }
  
  alltubes = exp->GetAllTubes();
  

  //Check to see if roots exist in destination sessions.  If they are, prompt the user to see
  //if they want them overwritten
  for (i=0; i<alltubes->size(); i++) 
  {
    roots = alltubes->at(i).GetAllRoots();
	m_roots = alltubes->at(i).GetAllRoots();
    
    //Loop through ALL roots in the current tube.
    for (j=0; j<roots->size(); j++)
    {
      if(copy_all || (roots->at(j).GetPrimaryWindow() == m_curr_window && tube->GetTag() == alltubes->at(i).GetTag()))
      {
        instances = roots->at(j).GetAllRootInstances();
		//m_roots = roots->at(j).GetAllRootInstances();
        //Loop through ALL sessions in the current root to determine if there already exists a session
        //in the selected session interval
        for( k=0; k<instances->size(); k++)
        {
          for( m= m_from+1; m <= m_to; m++)
          {
            // If the session number of this root instance is equal to l, which is in the interval from "from" to "to"
            //  ask the user if they want to overwrite the destination sessions.
            if( instances->at(k).GetSessionNumber() == m && !notified)
            {
              CString str;
              if (copy_all)
              {
                if (m_from+1 == m_to)
                {
                  str.Format("Session %d already contains roots.\r\nAre you sure you want to overwrite this data?", m_to);
                }
                else
                {
                  str.Format("Sessions %d through %d already contain roots.\r\nAre you sure you want to overwrite this data?", m_from+1, m_to);
                }
              }
              else
              {
                if (m_from+1 == m_to)
                {
                  str.Format("Window %d of Session %d already contains roots.\r\nAre you sure you want to overwrite this data?", m_curr_window, m_to);
                }
                else
                {
                  str.Format("Window %d of Sessions %d through %d already contain roots.\r\nAre you sure you want to overwrite this data?", m_curr_window, m_from+1, m_to);
                }
              }
              //DWORD choice = AfxMessageBox(str, MB_YESNO | MB_DEFBUTTON2);
              //if (choice != IDYES)  ok_to_copy = false;
              notified = true;
            }
          }
        }
      }
    }
  }


  //If the user chose to overwrite (or wasn't prompted at all) continue
  if (ok_to_copy) 
  {
    //Copy over all tubes and windows
    if (copy_all)
    {
      for (i=0; i<alltubes->size();i++)
      {
        roots = alltubes->at(i).GetAllRoots();
        for (j=0; j < roots->size(); j++) 
        {
          for (k=m_from+1; k <=m_to; k++) 
          {
            roots->at(j).DeleteRootInstance(k);
            //m_roots->at(j).CopyRootInstance(k,m_from);
          }
        }
      }      
    }
    //else only copy within the current window
    else 
    {
      roots = tube->GetAllRoots();
      for(int i = 0; i < roots->size(); i++)
      {
        if(roots->at(i).GetPrimaryWindow() == m_curr_window)
        {   
          for (k=m_from+1; k<=m_to;k++) 
          {
            roots->at(i).DeleteRootInstance(k);
            //m_roots->at(i).CopyRootInstance(k,m_from);
          }
        }
      }
    }
  }
  return true;
}
    